﻿using System.IO;

namespace TestProject
{
    /// <summary>
    /// 파일 시스템 도우미
    /// </summary>
    public class FileSystemHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 논리 드라이브 배열 구하기 - GetLogicalDriveArray()

        /// <summary>
        /// 논리 드라이브 배열 구하기
        /// </summary>
        /// <returns>논리 드라이브 배열</returns>
        public string[] GetLogicalDriveArray()
        {
            return Directory.GetLogicalDrives();
        }

        #endregion
        #region 디렉토리 경로 배열 구하기 - GetDirectoryPathArray(parentDirectoryPath)

        /// <summary>
        /// 디렉토리 경로 배열 구하기
        /// </summary>
        /// <param name="parentDirectoryPath">부모 디렉토리 경로</param>
        /// <returns>디렉토리 경로 배열</returns>
        public string[] GetDirectoryPathArray(string parentDirectoryPath)
        {
            return Directory.GetDirectories(parentDirectoryPath);
        }

        #endregion
        #region 파일 경로 배열 구하기 - GetFilePathArray(directoryPath)

        /// <summary>
        /// 파일 경로 배열 구하기
        /// </summary>
        /// <param name="directoryPath">디렉토리 경로</param>
        /// <returns>파일 경로 배열</returns>
        public string[] GetFilePathArray(string directoryPath)
        {
            return Directory.GetFiles(directoryPath);
        }

        #endregion
        #region 디렉토리명 구하기 - GetDirectoryName(directoryPath)

        /// <summary>
        /// 디렉토리명 구하기
        /// </summary>
        /// <param name="directoryPath">디렉토리 경로</param>
        /// <returns>디렉토리명</returns>
        public string GetDirectoryName(string directoryPath)
        {
            return new DirectoryInfo(directoryPath).Name;
        }

        #endregion
        #region 파일명 구하기 - GetFileName(filePath)

        /// <summary>
        /// 파일명 구하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>파일명</returns>
        public string GetFileName(string filePath)
        {
            return new FileInfo(filePath).Name;
        }

        #endregion
        #region 파일 크기 구하기 - GetFileSize(filePath)

        /// <summary>
        /// 파일 크기 구하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        /// <returns>파일 크기</returns>
        public long GetFileSize(string filePath)
        {
            return new FileInfo(filePath).Length;
        }

        #endregion
    }
}